/************************************************************
* Este es el fichero para crear la base de datos.
* 
*
* Fecha creacin: 13/03/2021
* 
* TFG: Creacin base de datos de procesos electorales.
*
* Autor: Francisco Jos Martnez del Amo
* Tutor: Jordi Ferrer Durn
************************************************************/

//Creacin de las tablas de auditoras y seguimientos

CREATE TABLE LogProcedimientos(
    ID_LP NUMBER CONSTRAINT PK_LogProcedimientos PRIMARY KEY,
    fechaHora DATE CONSTRAINT NN_LogProc_FechaHora NOT NULL,
    usuario VARCHAR2(20) CONSTRAINT NN_LogProc_Usuario NOT NULL,
    nombreProcedimiento VARCHAR2(50)  CONSTRAINT NN_LogProc_Nombre NOT NULL,
    paramEntrada VARCHAR2(4000) CONSTRAINT NN_LogProc_ParamEntrada NOT NULL,
    RSP VARCHAR2(100) CONSTRAINT NN_LogProc_ParamSalida NOT NULL
);

CREATE TABLE LogOperaciones(
    ID_LO NUMBER CONSTRAINT PK_LogOperaciones PRIMARY KEY,
    fechaHora DATE CONSTRAINT NN_LogOp_FechaHora NOT NULL,
    usuario VARCHAR2(20) CONSTRAINT NN_LogOp_Usuario NOT NULL,
    nombreEventos VARCHAR2(50)  CONSTRAINT NN_LogOp_Nombre NOT NULL,
    paramEntrada VARCHAR2(4000) CONSTRAINT NN_LogOp_ParamEntrada NOT NULL,
	paramSalida VARCHAR2(100),
    RSP VARCHAR2(100) CONSTRAINT NN_LogOp_ParamSalida NOT NULL
);


//Creacin de las tablas de integracin de datos

CREATE TABLE Persona(
    numDoc CHAR(8),
    letraDoc CHAR(1), 
    nombre VARCHAR2(30) CONSTRAINT NN_PersonaNombre NOT NULL,
    apellido1 VARCHAR2(30) CONSTRAINT NN_PersonaApellido1 NOT NULL,
    apellido2 VARCHAR2(30) CONSTRAINT NN_PersonaApellido2 NOT NULL,
    sexo VARCHAR2(10)
        CONSTRAINT NN_PersonaSexo NOT NULL
        CONSTRAINT ENUM_PersonaSexo CHECK (sexo IN('hombre','mujer','noBinario')),
    docIdentidad VARCHAR2(20)
        CONSTRAINT ENUM_PersonaDocIdentidad CHECK (docIdentidad IN('DNI','pasaporte','tarjetaExtranjeria'))
        CONSTRAINT NN_PersonaDocIdentidad NOT NULL,
    estudiosTerminados VARCHAR2(25)
        CONSTRAINT ENUM_PersonaEstudioTerminado CHECK (estudiosTerminados IN('sinEstudios','certificadoEscolaridad','ESO/EGB',
        'FP1/CFGM','FP2/CFGS','bachillerato','otrosTitulosMedios','diplomatura','grado','licenciado','master','doctorado'))
        CONSTRAINT NN_PersonaEstudiosTerminados NOT NULL,
    fechaNac DATE CONSTRAINT NN_PersonaFechaNac NOT NULL,
    municipio VARCHAR2(20) CONSTRAINT NN_PersonaMunicipioNac NOT NULL,
    provincia VARCHAR2(20) CONSTRAINT NN_PersonaProvinciaNac NOT NULL,
    nacionalidad VARCHAR2(20) CONSTRAINT NN_PersonaNacionalidad NOT NULL,
    CONSTRAINT PK_Persona PRIMARY KEY (numDoc, letraDoc)
);

CREATE TABLE Direccion(
    ID_Direccion INTEGER, 
    numDoc CHAR(8), 
    letraDoc CHAR(1), 
    entidadColectiva VARCHAR2(20), 
    entidadSingular VARCHAR2(20), 
	nucleo VARCHAR2(20), 
    tipoDir VARCHAR2(20), 
    nombreVia VARCHAR2(50), 
    numero CHAR(10), 
    letra CHAR(2), 
    km FLOAT, 
    bloque CHAR(2), 
    portal CHAR(10), 
    escalera CHAR(10), 
	piso CHAR(2), 
    puerta CHAR(2),
    CONSTRAINT PK_Direccion PRIMARY KEY (ID_Direccion, numDoc, letraDoc),
    CONSTRAINT FK_DireccionPersona FOREIGN KEY (numDoc, letraDoc) REFERENCES Persona(numDoc, letraDoc)
);

CREATE TABLE Telefono(
    numDoc CHAR(8),
    letraDoc CHAR(1), 
    ID_Telefono INTEGER,
    prefijo CHAR(3) CONSTRAINT NN_TelefonoPrefijo NOT NULL,
    numTelefono INTEGER CONSTRAINT NN_TelefonoNumTelefono NOT NULL,
    CONSTRAINT PK_Telefono PRIMARY KEY (numDoc, letraDoc, ID_Telefono),
    CONSTRAINT FK_TelefonoPersona FOREIGN KEY (numDoc, letraDoc) REFERENCES Persona(numDoc, letraDoc)
);

//Creacin de las tablas de proceso electoral

CREATE TABLE ReglaProceso(
    ID_RP INTEGER CONSTRAINT PK_ReglaProceso PRIMARY KEY, 
    nombreRP VARCHAR2(20) CONSTRAINT NN_ReglaProcesoNombreRP NOT NULL, 
    descripcionRP VARCHAR2(200) CONSTRAINT NN_ReglaProcesoDescripcionRP NOT NULL
);

CREATE TABLE Gobierno (
    ID_Gobierno INTEGER CONSTRAINT PK_Gobierno PRIMARY KEY, 
    ambito VARCHAR2(11) 
        CONSTRAINT NN_GobiernoAmbito NOT NULL
        CONSTRAINT ENUM_GobiernoAmbito CHECK(ambito IN('General','Autonmico','Local')),  
    nombreGob VARCHAR(40)
        CONSTRAINT NN_GobiernoNombreGob NOT NULL
        CONSTRAINT U_GobiernoNombreGob UNIQUE
);

CREATE TABLE ProcesoElectoral (
    ID_Proceso INTEGER CONSTRAINT PK_ProcesoElectoral PRIMARY KEY, 
    ambito VARCHAR2(11) 
        CONSTRAINT NN_PE_Ambito NOT NULL
        CONSTRAINT ENUM_PE_Ambito CHECK(ambito IN('General','Autonmico','Local')), 
    fechaElecciones DATE CONSTRAINT NN_PE_FechaElecciones NOT NULL, 
	nombreProceso VARCHAR2(80) CONSTRAINT NN_PE_Nombre NOT NULL, 
    ID_Gobierno INTEGER CONSTRAINT NN_PE_ID_Gobierno NOT NULL,
    CONSTRAINT FK_PE_Gobierno FOREIGN KEY (ID_Gobierno) REFERENCES Gobierno(ID_Gobierno)
);

CREATE TABLE DemarcacionElectoral (
    ID_Demarcacion NUMBER CONSTRAINT PK_DemarcacionElectoral PRIMARY KEY, 
    nombreDemarcacion VARCHAR2(50) CONSTRAINT NN_DE_NombreDemarcacion NOT NULL, 
    numEscanos INTEGER CONSTRAINT NN_DE_NumEscanos NOT NULL,
 	ID_Proceso INTEGER CONSTRAINT NN_DE_ID_Proceso NOT NULL,
    CONSTRAINT FK_DE_ProcesoElectoral FOREIGN KEY (ID_Proceso) REFERENCES ProcesoElectoral(ID_Proceso)
);

CREATE TABLE Provincia (
    ID_Provincia INTEGER CONSTRAINT PK_Provincia PRIMARY KEY, 
    nombrePro VARCHAR2(50) 
        CONSTRAINT U_ProvinciaNombre UNIQUE
        CONSTRAINT NN_ProvinciaNombre NOT NULL,
	escanosGen INTEGER CONSTRAINT NN_ProvinciaEscanosGen NOT NULL,
	escanosAut INTEGER CONSTRAINT NN_ProvinciaEscanosAut NOT NULL
);

CREATE TABLE Localidad (
	ID_Provincia INTEGER, 
    ID_Localidad INTEGER, 
    nombreLoc VARCHAR2(50) 
        CONSTRAINT U_LocalidadNombreLoc UNIQUE
        CONSTRAINT NN_LocalidadNombreLoc NOT NULL,
	escanosLoc INTEGER CONSTRAINT NN_LocalidadEscanosLoc NOT NULL,
	CONSTRAINT PK_Localidad PRIMARY KEY (ID_Provincia, ID_Localidad),
	CONSTRAINT FK_ProvinciaLocalidad FOREIGN KEY (ID_Provincia) REFERENCES Provincia(ID_Provincia)
);
	
CREATE TABLE CentroVotacion (
    ID_Centro NUMBER CONSTRAINT PK_CentroVotacion PRIMARY KEY, 
    nombreCentro VARCHAR2(50) CONSTRAINT NN_CV_NombreCentro NOT NULL, 
    direccionCentro VARCHAR2(100) CONSTRAINT NN_CV_DireccionCentro NOT NULL, 
    ID_Provincia INTEGER CONSTRAINT NN_CV_ID_Provincia NOT NULL,
    ID_Localidad INTEGER CONSTRAINT NN_CV_ID_Localidad NOT NULL,
    CONSTRAINT FK_CV_Localidad FOREIGN KEY (ID_Provincia, ID_Localidad) REFERENCES Localidad(ID_Provincia, ID_Localidad),
    ID_Demarcacion NUMBER CONSTRAINT NN_LocalidadID_Demarcacion NOT NULL,
    CONSTRAINT FK_CV_DE FOREIGN KEY (ID_Demarcacion) REFERENCES DemarcacionElectoral(ID_Demarcacion)
);
	
CREATE TABLE MesaElectoral (
    ID_Mesa NUMBER CONSTRAINT PK_MesaElectoral PRIMARY KEY, 
    nombreMesa VARCHAR2(10) CONSTRAINT NN_MesaElectoralNombreMesa NOT NULL, 
    ID_Centro NUMBER CONSTRAINT NN_MedaElectoralID_Centro NOT NULL,
    CONSTRAINT FK_MesaElectoralCentroVotacion FOREIGN KEY (ID_Centro) REFERENCES CentroVotacion(ID_Centro)
);

CREATE TABLE Voto(
    ID_Voto NUMBER CONSTRAINT PK_Voto PRIMARY KEY, 
    modalidad VARCHAR2(10)
        CONSTRAINT NN_VotoModalidad NOT NULL
        CONSTRAINT ENUM_VotoModalidad CHECK (modalidad IN('presencial','telematico')), 
    partido VARCHAR2(50), 
    tipoVoto VARCHAR2(10)
        CONSTRAINT NN_VotoTipoVoto NOT NULL
        CONSTRAINT ENUM_VotoTipoVoto CHECK (tipoVoto IN('normal','nulo','enBlanco')), 
    ID_Mesa NUMBER CONSTRAINT NN_VotoID_Mesa NOT NULL,
    CONSTRAINT FK_VotoMesaElectoral FOREIGN KEY (ID_Mesa) REFERENCES MesaElectoral(ID_Mesa)
);

CREATE TABLE CensoElectoral (
    numDoc CHAR(8),
    letraDoc CHAR(1),  
    ID_RP VARCHAR2(30), 
    ID_Proceso INTEGER, 
    fechaElecciones DATE CONSTRAINT NN_CE_FechaElecciones NOT NULL, 
    inscrito CHAR(1)
        CONSTRAINT B_CE_Inscrito CHECK (inscrito IN('S','N'))
        CONSTRAINT NN_CE_Inscrito NOT NULL, 
	mesaElectoral NUMBER CONSTRAINT NN_CE_MesaElectoral NOT NULL, 
    centroVotacion VARCHAR2(50) CONSTRAINT NN_CE_CentroVotacion NOT NULL, 
    ID_Voto NUMBER,
    CONSTRAINT PK_CensoElectoral PRIMARY KEY (numDoc, letraDoc, ID_RP, ID_Proceso),
    CONSTRAINT FK_CE_Persona FOREIGN KEY (numDoc, letraDoc) REFERENCES Persona(numDoc, letraDoc),
    CONSTRAINT FK_CE_ProcesoElectoral FOREIGN KEY (ID_Proceso) REFERENCES ProcesoElectoral(ID_Proceso),
    CONSTRAINT FK_CE_Voto FOREIGN KEY (ID_Voto) REFERENCES Voto(ID_Voto)
);

CREATE TABLE ComunicacionVotantes (
    ID_CV NUMBER CONSTRAINT PK_ComunicacionVotantes PRIMARY KEY, 
    numDoc CHAR(8),
    letraDoc CHAR(1), 
    mesaElectoral VARCHAR2(10) CONSTRAINT NN_CV_MesaElectoral NOT NULL, 
    centroVotacion VARCHAR2(50) CONSTRAINT NN_CV_CentroVotacion NOT NULL, 
    instruccionesCV VARCHAR2(2000) CONSTRAINT NN_CV_InstruccionesCV NOT NULL, 
	ID_Proceso INTEGER CONSTRAINT NN_CV_ID_Proceso NOT NULL,
    CONSTRAINT FK_CV_Persona FOREIGN KEY (numDoc, letraDoc) REFERENCES Persona(numDoc, letraDoc),
    CONSTRAINT FK_CV_ProcesoElectoral FOREIGN KEY (ID_Proceso) REFERENCES ProcesoElectoral(ID_Proceso)
);

CREATE TABLE ReglasPartidosPoliticos (
    ID_RPP INTEGER CONSTRAINT PK_ReglasPartidosPoliticos PRIMARY KEY,
    nombreRPP VARCHAR2(20) CONSTRAINT NN_RPP_NompreRPP NOT NULL,
    descripcionRPP VARCHAR2(150) CONSTRAINT NN_RPP_DescripcionRPP NOT NULL
);

CREATE TABLE PartidoPolitico (
    ID_Partido INTEGER CONSTRAINT PK_PartidoPolitico PRIMARY KEY, 
    nombrePartido VARCHAR2(50) 
        CONSTRAINT U_PartidoNombre UNIQUE
        CONSTRAINT NN_PP_NombrePartido NOT NULL,
    siglas VARCHAR2(15) CONSTRAINT NN_PP_Siglas NOT NULL
);

CREATE TABLE ParticipaEn (
    ID_Proceso INTEGER, 
    ID_RPP INTEGER, 
    ID_Partido INTEGER,
    CONSTRAINT FK_PE_ProcesoElectoral FOREIGN KEY (ID_Proceso) REFERENCES ProcesoElectoral(ID_Proceso),
    CONSTRAINT FK_PE_ReglasPartidosPoliticos FOREIGN KEY (ID_RPP) REFERENCES ReglasPartidosPoliticos(ID_RPP),
    CONSTRAINT FK_PE_PartidoPolitico FOREIGN KEY (ID_Partido) REFERENCES PartidoPolitico(ID_Partido)
);

CREATE TABLE ComunicacionBOE (
    ID_ComBOE INTEGER CONSTRAINT PK_ComunicacionBOE PRIMARY KEY, 
    ambito VARCHAR2(10) 
        CONSTRAINT NN_CBOEAmbito NOT NULL
        CONSTRAINT ENUM_CBOEAmbito CHECK(ambito IN('General','Autonmico','Local')), 
    fechaElecciones DATE CONSTRAINT NN_CBOEFechaElecciones NOT NULL, 
    ReglasPartidos VARCHAR2(4000) CONSTRAINT NN_cBOEReglasPartidos NOT NULL, 
	metodoCalculo VARCHAR2(50) CONSTRAINT NN_CBOEMetodoCalculo NOT NULL, 
    demarcacion  VARCHAR2(4000) CONSTRAINT NN_CBOEDemarcacion NOT NULL, 
    escanosNum INTEGER CONSTRAINT NN_CBOEEscanosNum NOT NULL, 
    ID_Gobierno INTEGER CONSTRAINT NN_CBOEID_Gobierno NOT NULL,
    CONSTRAINT FK_CBOEGobierno FOREIGN KEY (ID_Gobierno) REFERENCES Gobierno(ID_Gobierno)
);

CREATE TABLE ListaCandidatos (
    ID_Lista INTEGER CONSTRAINT PK_ListaCandidatos PRIMARY KEY, 
    aprobada CHAR(1)
        CONSTRAINT B_LC_Aprobada CHECK (aprobada IN('S','N'))
        CONSTRAINT NN_LC_Aprobada NOT NULL, 
    ID_Partido INTEGER CONSTRAINT NN_LC_ID_Partido NOT NULL, 
    ID_Demarcacion NUMBER CONSTRAINT NN_LC_Demarcacion NOT NULL, 
    ID_Gobierno INTEGER CONSTRAINT NN_LC_Gobierno NOT NULL,
    CONSTRAINT FK_LC_PartidoPolitico FOREIGN KEY (ID_Partido) REFERENCES PartidoPolitico(ID_Partido),
    CONSTRAINT FK_LC_DemarcacionElectoral FOREIGN KEY (ID_Demarcacion) REFERENCES DemarcacionElectoral(ID_Demarcacion),
    CONSTRAINT FK_LC_Gobierno FOREIGN KEY (ID_Gobierno) REFERENCES Gobierno(ID_Gobierno)    
);

CREATE TABLE Candidato (
    numDoc CHAR(8),
    letraDoc CHAR(1), 
    ID_Partido INTEGER,
    nombre VARCHAR2(20) CONSTRAINT NN_CandidatoNombre NOT NULL, 
    apellido1 VARCHAR2(20) CONSTRAINT NN_CandidatoApellido1 NOT NULL, 
    apellido2 VARCHAR2(20) CONSTRAINT NN_CandidatoApellido2 NOT NULL, 
    ID_Lista INTEGER,
    CONSTRAINT PK_Candidato PRIMARY KEY (numDoc, letraDoc, ID_Partido),
    CONSTRAINT FK_CandidatoPartidoPolitico FOREIGN KEY (ID_Partido) REFERENCES PartidoPolitico(ID_Partido),
    CONSTRAINT FK_CandidatoListaCandidatos FOREIGN KEY (ID_Lista) REFERENCES ListaCandidatos(ID_Lista)
);

CREATE TABLE Recuento (
    ID_Recuento NUMBER CONSTRAINT PK_Recuento PRIMARY KEY, 
    nulos INTEGER CONSTRAINT NN_RecuentoNulos NOT NULL,
    enBlanco INTEGER CONSTRAINT NN_RecuentosEnBlanco NOT NULL,
    ID_Mesa NUMBER CONSTRAINT NN_RecuentosID_Mesa NOT NULL,
    CONSTRAINT FK_RecuentoMesaElectoral FOREIGN KEY (ID_Mesa) REFERENCES MesaElectoral(ID_Mesa)
);

CREATE TABLE Desglose (
    ID_Recuento NUMBER,
    ID_Desglose INTEGER,
    nombrePartido VARCHAR2(50), 
    totalVotos INTEGER, 
    CONSTRAINT PK_Desglose PRIMARY KEY (ID_Recuento, ID_Desglose),
    CONSTRAINT FK_DesgloseRecuento FOREIGN KEY (ID_Recuento) REFERENCES Recuento(ID_Recuento)
);

CREATE TABLE MetodoCalculo (
    ID_Metodo INTEGER CONSTRAINT PK_MetodoCalculo PRIMARY KEY, 
    nombreMetodo VARCHAR2(50) CONSTRAINT NN_MetodoCalculoNombreMetodo NOT NULL
);

CREATE TABLE Resultado (
    ID_Demarcacion NUMBER, 
    ID_Recuento NUMBER, 
    ID_Metodo INTEGER, 
    fechaElecciones DATE CONSTRAINT NN_R_FechaElecciones NOT NULL,
    CONSTRAINT PF_Resultado PRIMARY KEY (ID_Demarcacion, ID_Recuento, ID_Metodo),
    CONSTRAINT FK_R_DemarcacionElectoral FOREIGN KEY (ID_Demarcacion) REFERENCES DemarcacionElectoral(ID_Demarcacion),
    CONSTRAINT FK_R_Recuento FOREIGN KEY (ID_Recuento) REFERENCES Recuento(ID_Recuento),
    CONSTRAINT FK_R_MetodoCalculo FOREIGN KEY (ID_Metodo) REFERENCES MetodoCalculo(ID_Metodo)
);

CREATE TABLE ReglasSeleccion(
    ID_RS INTEGER CONSTRAINT PK_ReglasSeleccion PRIMARY KEY, 
    nombreRS VARCHAR2(50) CONSTRAINT NN_RS_NombreRS NOT NULL, 
    descripcionRS VARCHAR2(200) CONSTRAINT NN_RS_DescripcionRS NOT NULL
);

CREATE TABLE MiembroMesa(
    numDoc CHAR(8),
    letraDoc CHAR(1),  
    ID_RP VARCHAR2(30), 
    ID_Proceso INTEGER,
    ID_RS VARCHAR2(30), 
    ID_Mesa NUMBER, 
	fechaElecciones DATE CONSTRAINT NN_MM_FechaElecciones NOT NULL, 
    cargo VARCHAR2(10)
        CONSTRAINT NN_MM_Cargo NOT NULL
        CONSTRAINT ENUM_MM_Cargo CHECK (cargo IN('presidente','secretario','vocal','suplente')), 
    localizado CHAR(1)
        CONSTRAINT B_MM_Localizado CHECK (localizado IN('S','N'))
        CONSTRAINT NN_MM_Localizado NOT NULL, 
    alegaciones CHAR(1)
        CONSTRAINT B_MM_Alegaciones CHECK (alegaciones IN('S','N'))
        CONSTRAINT NN_MM_Alegaciones NOT NULL, 
    apto CHAR(1)
        CONSTRAINT B_MM_Apto CHECK (apto IN('S','N'))
        CONSTRAINT NN_MM_Apto NOT NULL, 
    CONSTRAINT PK_MiembroMesa PRIMARY KEY (numDoc, letraDoc, ID_RP, ID_Proceso, ID_RS, ID_Mesa),
    CONSTRAINT FK_MM_CensoElectoral FOREIGN KEY (numDoc, letraDoc, ID_RP, ID_Proceso) 
        REFERENCES CensoElectoral(numDoc, letraDoc, ID_RP, ID_Proceso),
    CONSTRAINT FK_MM_MesaElectoral FOREIGN KEY (ID_Mesa) REFERENCES MesaElectoral(ID_Mesa)
);

CREATE TABLE muestraHondt (
    partido VARCHAR2(50), 
    votos INTEGER, 
    escanos INTEGER
    ); 
    
CREATE TABLE muestraSainte (
    partido VARCHAR2(50), 
    votos INTEGER, 
    escanos INTEGER
    ); 

//Creacin de secuencias
@.\secuenciasBD.sql

//Creacin procedimientos CRUD
@.\procedimientosCRUD.sql

//Creacin procedimientos
@.\procedimientosBD.sql

//Creacin triggers

//Creacin de vistas
CREATE VIEW erroresProcedimientosABM
AS
SELECT *
FROM LogProcedimientos 
WHERE RSP NOT LIKE 'OK';

CREATE VIEW erroresOperaciones
AS
SELECT *
FROM LogOperaciones 
WHERE RSP NOT LIKE 'OK';