/************************************************************
* Este es el conjunto mnimo bsico inicial de datos.
* Permite verificar las funciones mnimas.
* 
* Los datos de Personas deben realizarse con un proceso de carga de datos.
*
* Se realiza la inicializacin de las secuencias correspondientes.
* Se procede a dar de alta un proceso electoral completo en Zaragoza
* que simula las elecciones municipales del 2019.
*
* Fecha creacin: 22/03/2021
* 
* TFG: Creacin base de datos de procesos electorales.
*
* Autor: Francisco Jos Martnez del Amo
* Tutor: Jordi Ferrer Durn
************************************************************/

//Inicializamos la secuencia de los log de errores
SELECT Sec_LogProcedimientos.NEXTVAL FROM dual;
SELECT Sec_LogOperaciones.NEXTVAL FROM dual;
SELECT Sec_Direccion.NEXTVAL FROM dual;
SELECT Sec_Telefono.NEXTVAL FROM dual;

// Primero generamos los gobiernos que inician los procesos electorales
SELECT Sec_Gobierno.NEXTVAL FROM dual;
DECLARE
RSP VARCHAR2(500);
BEGIN
altaGobierno (Sec_Gobierno.CURRVAL, 'General', 'Gobierno de Espaa', RSP);
altaGobierno (Sec_Gobierno.NEXTVAL, 'Autonmico', 'Gobierno de Aragn', RSP);
altaGobierno (Sec_Gobierno.NEXTVAL, 'Local', 'Ayuntamiento de Zaragoza', RSP);
END;
/

//El Gobierno inicia un proceso
SELECT Sec_ProcesoElectoral.NEXTVAL FROM dual;
DECLARE
RSP VARCHAR2(500);
BEGIN
altaProcesoElectoral (Sec_ProcesoElectoral.CURRVAL, 'General', '10/11/2019', 'Elecciones generales de noviembre de 2019', 1, RSP);
altaProcesoElectoral (Sec_ProcesoElectoral.NEXTVAL, 'Autonmico', '26/05/2019', 'Elecciones a las Cortes de Aragn 2019', 2, RSP);
altaProcesoElectoral (Sec_ProcesoElectoral.NEXTVAL, 'Local', '26/05/2019', 'Elecciones al Ayuntamiento de Zaragoza', 3, RSP);
END;
/

//Cargamos datos de las provincias
@.\provincias.sql

//Cargamos los datos de las localidades
@.\localidades.sql

//Cargamos los partidos polticos
SELECT Sec_PartidoPolitico.NEXTVAL FROM dual;
@.\partidos.sql
@.\partidosZGZ.sql

//Cargamos los mtodos de clculo
SELECT Sec_MetodoCalculo.NEXTVAL FROM dual;
DECLARE
RSP VARCHAR2(500);
BEGIN
altaMetodoCalculo (Sec_MetodoCalculo.CURRVAL, 'Sistema D.Hondt', RSP);
altaMetodoCalculo (Sec_MetodoCalculo.NEXTVAL, 'Mtodo Sainte-Lagu', RSP);
altaMetodoCalculo (Sec_MetodoCalculo.NEXTVAL, 'Cociente Droop', RSP);
END;
/

//Cargamos las reglas genricas para los procesos electorales
SELECT Sec_ReglaProceso.NEXTVAL FROM dual;
DECLARE
RSP VARCHAR2(500);
BEGIN
altaReglaProceso (Sec_ReglaProceso.CURRVAL, 'Mayoria de edad', 'La persona debe ser mayor de 18 aos', RSP);
altaReglaProceso (Sec_ReglaProceso.NEXTVAL, 'Nacionalidad', 'La persona debe tener la nacionalidad Espaola', RSP);
altaReglaProceso (Sec_ReglaProceso.NEXTVAL, 'Union Europea', 'La persona debe tener la nacionalidad Espaola o de un pais miembro de la Unin Europea', RSP);
altaReglaProceso (Sec_ReglaProceso.NEXTVAL, 'Empadronado', 'La persona debe estar empadronado en el municipio correspondiente a la demarcacin electoral.', RSP);
END;
/

//Cargamos las reglas genricas para la partipacin de los partidos polticos
SELECT Sec_ReglasPartidosPoliticos.NEXTVAL FROM dual;
DECLARE
RSP VARCHAR2(500);
BEGIN
altaReglasPartidosPoliticos (Sec_ReglasPartidosPoliticos.CURRVAL, 'Inscripcin', 'El partido debe estar inscrito, es decir en la base de datos', RSP);
altaReglasPartidosPoliticos (Sec_ReglasPartidosPoliticos.NEXTVAL, 'Candidatos', 'El partido debe tener suficientes candidatos', RSP);
END;
/

//Cargamos los partidos en el proceso electoral de Zaragoza
@.\participaEn.sql

//Creamos las demarcaciones en funcin del Proceso Electoral, en este caso para el proceso de elecciones municipales en Zaragoza
//Una vez creadas aadimos los centros de votacin y las respectivas mesas, siempre para el proceso de elecciones municipales de Zaragoza
SELECT Sec_DemarcacionElectoral.NEXTVAL FROM dual;
SELECT Sec_CentroVotacion.NEXTVAL FROM dual;
SELECT Sec_MesaElectoral.NEXTVAL FROM dual;
DECLARE
paramSalida VARCHAR2(500);
RSP VARCHAR2(500);
BEGIN
crearDemarcaciones (3, 'Zaragoza', paramSalida, RSP);
/* Creamos los centros de votacin y las Mesas de cada centro para esa demarcacin */
crearCentrosCiudad (2, 50, 297, paramSalida, RSP);
END;
/

//Con todas las mesas creadas, el gobierno realiza la comunicacin al BOE empezando el proceso y 
//se asigna el censo y las comunicaciones a los votantes
SELECT Sec_ComunicacionBOE.NEXTVAL FROM dual;
SELECT Sec_ComunicacionVotantes.NEXTVAL FROM dual;
DECLARE
paramSalida VARCHAR2(4000);
RSP VARCHAR2(500);
BEGIN
crearComBOE(3, paramSalida, RSP);
crearCenso(3, paramSalida, RSP);
crearComVot(3, paramSalida, RSP);
END;
/

// Ahora cargamos los candidatos y la lista de candidatos
SELECT Sec_ListaCandidatos.NEXTVAL FROM dual;
@.\listacandidatos.sql
@.\candidatos.sql

SELECT Sec_ReglasSeleccion.NEXTVAL FROM dual;
DECLARE
RSP VARCHAR2(500);
BEGIN
altaReglasSeleccion (Sec_MetodoCalculo.CURRVAL, 'leer', 'Saber leer y escribir', RSP);
altaReglasSeleccion (Sec_MetodoCalculo.NEXTVAL, 'edad', 'Ser menor de 70 aos', RSP);
altaReglasSeleccion (Sec_MetodoCalculo.NEXTVAL, 'candidato', 'No ser candidato', RSP);
END;
/

//Seleccionamos los miembros de cada mesa, con valores por defecto que modificaremos posteriormente
DECLARE
RSP VARCHAR2(500);
paramSALIDA VARCHAR2(500);
BEGIN
seleccionarMiembros(3, paramSalida, RSP);
simularEstados(3, paramSalida, RSP); 
END;
/

SELECT Sec_Voto.NEXTVAL FROM dual;
DECLARE
RSP VARCHAR2(500);
paramSALIDA VARCHAR2(500);
BEGIN
asignarVotos2019(3, paramSalida, RSP);
END;
/

/* hacemos el recuento de las elecciones */
SELECT Sec_Recuento.NEXTVAL FROM dual;
SELECT Sec_Desglose.NEXTVAL FROM dual;
DECLARE
RSP VARCHAR2(500);
paramSALIDA VARCHAR2(4000);
BEGIN
realizaRecuento(3, paramSalida, RSP);
END;
/

/* aadimos la tabla de resultados y calculamos los escaos obtenidos */
DROP TABLE muestraHondt;
CREATE TABLE muestraHondt AS (
    SELECT nombrePartido AS PARTIDO, sum(totalVotos) AS VOTOS, 0 as ESCANOS
    FROM Desglose
    WHERE ID_Recuento IN (
        SELECT r.ID_Recuento 
        FROM Recuento r
        JOIN MesaElectoral m ON r.ID_Mesa = m.Id_Mesa
        JOIN CentroVotacion c ON m.ID_Centro = c.ID_Centro
        JOIN DemarcacionElectoral d ON c.ID_Demarcacion = 2)
    GROUP BY nombrePartido); 
DECLARE
RSP VARCHAR2(500);
paramSALIDA VARCHAR2(4000);
BEGIN
listarResultados(3, 1, paramSalida, RSP);
/* creamos la tabla por esa demarcacin para obtener una foto de los resultados y no calcularlo cada vez */
calculaHondt(3, 2, paramSalida, RSP);
END;
/

/* Calculamos por Sainte slo para ver la diferencia entre un sistema y otro */
/* creamos la tabla por esa demarcacin para obtener una foto de los resultados y no calcularlo cada vez */
DROP TABLE muestraSainte;
CREATE TABLE muestraSainte AS (
    SELECT nombrePartido AS PARTIDO, sum(totalVotos) AS VOTOS, 0 as ESCANOS
    FROM Desglose
    WHERE ID_Recuento IN (
        SELECT r.ID_Recuento 
        FROM Recuento r
        JOIN MesaElectoral m ON r.ID_Mesa = m.Id_Mesa
        JOIN CentroVotacion c ON m.ID_Centro = c.ID_Centro
        JOIN DemarcacionElectoral d ON c.ID_Demarcacion = 2)
    GROUP BY nombrePartido); 
DECLARE
RSP VARCHAR2(500);
paramSALIDA VARCHAR2(4000);
BEGIN
calculaSainte(3, 2, paramSalida, RSP);
END;
/

COMMIT;


