
/************************************************************
* Procedimiento para dar de alta un Censo Electoral en función
* del ámbito del Proceso Electoral a la hora de aplicar unas
* reglas determinadas.
*
* Ambito General: Reglas mayoría de edad y nacionalidad española
* Ámbito Autonómico: Edad, nacionalidad y empadronamiento
* Ámbito Local: Edad, nacionalidad Europea y empadronamiento
*
* Entrada: ID_Proceso
* RSP salida: OK si se realiza con éxito
*             ERROR+TIPO DE ERROR en caso contrario
* Salida: Número de personas insertadas, Número de personas posibles 
*         y número de fallos
* Pre-condición: el proceso electoral, las reglas y las personas deben existir.
* Post-condición: se crea el censo electoral de ese proceso.
************************************************************/

CREATE OR REPLACE PROCEDURE crearCenso(
	P_IDProceso IN ProcesoElectoral.ID_Proceso%TYPE,
    paramSalida OUT VARCHAR2,
	RSP OUT VARCHAR2) 
IS
    paramEntrada VARCHAR2(500);
	numERROR INTEGER;
	tipoERROR VARCHAR2(100);
    VACIO_ERROR EXCEPTION;
    procesoExiste ProcesoElectoral.ID_Proceso%TYPE;
    P_centroNum INTEGER;
    P_ambito ProcesoElectoral.ambito%TYPE;
    P_fecha ProcesoElectoral.fechaElecciones%TYPE;
    P_centro CentroVotacion.ID_Centro%TYPE;
    P_mesa MesaElectoral.ID_Mesa%TYPE;
    P_demarcacion DemarcacionElectoral.ID_Demarcacion%TYPE;
	totalInserciones NUMBER;
	totalPersonas NUMBER;
	totalFallos NUMBER;
	cuentaCentro INTEGER;
	cuentaMesa INTEGER;
    
BEGIN
    paramEntrada:= P_IDProceso;
	
    /* Comprobamos que el ID no sea nulo */
    IF (P_IDProceso IS NULL)
    THEN
        RAISE VACIO_ERROR;
    END IF;  
    
    /* Comprobamos que exista el Proceso Electoral */ 
    BEGIN
        SELECT ID_Proceso
        INTO procesoExiste
        FROM ProcesoElectoral
        WHERE ID_Proceso = P_IDProceso
        AND ROWNUM = 1;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
            RAISE NO_DATA_FOUND;
    END;
    
    /* seleccionamos datos del proceso para el alta */
    SELECT ambito, fechaElecciones
    INTO P_ambito, P_fecha
    FROM ProcesoElectoral
    WHERE ID_Proceso = P_IDProceso;
        
    /* Si ha pasado las comprobaciones anteriores, inscribimos en el censo */   
    IF (P_ambito = 'General')
    THEN   
		/* Pendiende de realizar para cuando se haga un ejemplo de votación general */
        /* Realizamos una inscripción por persona */
        altaCensoElectoral (1, 1, 1, P_IDProceso, P_fecha, 'S', 30, 29, NULL, RSP);
        
    ELSIF (P_ambito = 'Local') THEN

        /* Seleccionamos la Demarcación */
        SELECT Id_Demarcacion
        INTO P_demarcacion
        FROM DemarcacionElectoral
        WHERE ID_Proceso = P_IDProceso;
		
		/* Seleccionamos el centro de votación inicial */
		SELECT ID_Centro
		INTO P_centro
		FROM CentroVotacion
		WHERE ID_Demarcacion = P_demarcacion
		AND ROWNUM = 1
		ORDER BY ID_Centro;
		
		/* Seleccionamos la Mesa */
        SELECT Id_Mesa
        INTO P_mesa
        FROM MesaElectoral
        WHERE ID_Centro = P_centro
		AND ROWNUM = 1
		ORDER BY Id_Mesa;
        
		/* Ámbito Local: Edad, nacionalidad Europea y empadronamiento */
		totalInserciones:= 0;
        cuentaCentro:= 0;
        cuentaMesa:= 0;
		
		SELECT COUNT(*)
        INTO totalPersonas
        FROM Persona;
		
        DECLARE
        CURSOR C_personas IS
            SELECT numDoc, letraDoc 
            FROM Persona
            WHERE municipioNac='zaragoza'
            AND (months_between(SYSDATE,FECHANAC)/12)>17
            AND NACIONALIDAD IN ('Española','Europea');
        BEGIN
			FOR p IN C_personas
            LOOP
            /* realizamos la inscripción en el Censo Electoral */
            
			altaCensoElectoral (p.numDoc, p.letraDoc, 1, P_IDProceso, P_fecha, 'S', P_mesa, P_centro, NULL, RSP);
			totalInserciones:= totalInserciones+1;
			cuentaCentro:= cuentaCentro+1;
			cuentaMesa:= cuentaMesa+1;
            
			/* actualizamos Centro */
			IF (cuentaCentro = 20000)
			THEN
				P_centro:=P_centro+1;
			END IF; 
            
			/* actualizamos Mesa */
			IF (cuentaMesa = 2000)
			THEN
				P_mesa:=P_mesa+1;
			END IF;  
			
            END LOOP;
        END;
        
        /* seleccionamos el centro de votación */

    ELSE 
		/* Pendiende de realizar para cuando se haga un ejemplo de votación autonomica */
        /* Realizamos una inscripción por persona */
        altaCensoElectoral (1, 1, 1, P_IDProceso, P_fecha, 'S', 30, 29, NULL, RSP);
        
    END IF;
	
	/* Contamos los fallos */
	SELECT COUNT(*)
	INTO totalFallos
	FROM CensoElectoral
	WHERE ID_Proceso = P_IDProceso;
	totalFallos:= totalInserciones-totalFallos;
	
    paramSalida:='Total Censados: '||totalInserciones||' de un total de '||totalPersonas||' personas'||' y fallos: '||totalFallos;
    RSP:= 'OK';
    COMMIT;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
   
EXCEPTION
    WHEN VACIO_ERROR THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL VALOR DEL PROCEDIMEINTO NO PUEDE SER NULO';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN NO_DATA_FOUND THEN
    numERROR:= SQLCODE;
    RSP:= TO_CHAR(numERROR)||' - EL PROCEDIMIENTO NO SE ENCUENTRA - NO EXISTE';
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, paramSalida, RSP);
    COMMIT;
    
    WHEN OTHERS THEN
    numERROR:= SQLCODE;
    tipoERROR:= SUBSTR(SQLERRM, 1, 90);
    RSP := TO_CHAR(numERROR)||' - '||tipoERROR;
    ROLLBACK;
    INSERT INTO LogOperaciones VALUES(Sec_LogProcedimientos.NEXTVAL, SYSDATE, USER, $$PLSQL_UNIT, paramEntrada, PARAMsALIDA, RSP);
    COMMIT;
END;
/
